/*
 *  Copyright  2006-2011 SplinterGU (Fenix/Bennugd)
 *  Copyright  2002-2006 Fenix Team (Fenix)
 *  Copyright  1999-2002 Jos Luis Cebrin Page (Fenix)
 *
 *  This file is part of Bennu - Game Development
 *
 *  This software is provided 'as-is', without any express or implied
 *  warranty. In no event will the authors be held liable for any damages
 *  arising from the use of this software.
 *
 *  Permission is granted to anyone to use this software for any purpose,
 *  including commercial applications, and to alter it and redistribute it
 *  freely, subject to the following restrictions:
 *
 *     1. The origin of this software must not be misrepresented; you must not
 *     claim that you wrote the original software. If you use this software
 *     in a product, an acknowledgment in the product documentation would be
 *     appreciated but is not required.
 *
 *     2. Altered source versions must be plainly marked as such, and must not be
 *     misrepresented as being the original software.
 *
 *     3. This notice may not be removed or altered from any source
 *     distribution.
 *



 DC Bennu Port by Colombian Developers
 -------------------------------------
 */

/*
 * INCLUDES
 */

#include "bgdrtm.h"

#ifdef _WIN32
#define  _WIN32_WINNT 0x0500
#include <windows.h>
#endif


#include <time.h>

#include "bgdi.h"

#include "xstrings.h"
#include "dirs.h"

#if defined(TARGET_IOS)
#include <SDL.h>
#elif defined(TARGET_WII)
#include <SDL.h>
#include <fat.h>
#elif defined(TARGET_PSP)
#include "psp.h"
#elif defined(TARGET_ANDROID)
#include <SDL.h>
#endif

/* ---------------------------------------------------------------------- */

static char * dcb_exts[] = { ".dcb", ".dat", ".bin", NULL };

static int standalone  = 0;  /* 1 only if this is an standalone interpreter   */
static int embedded    = 0;  /* 1 only if this is a stub with an embedded DCB */

/* ---------------------------------------------------------------------- */

/*
 *  FUNCTION : main
 *
 *  Main function
 *
 *  PARAMS:
 *      INT n: ERROR LEVEL to return to OS
 *
 *  RETURN VALUE:
 *      No value
 *
 */

#include "png.h"

#include <kos.h>
KOS_INIT_FLAGS(INIT_DEFAULT);

#include "SDL.h"
#include "SDL_dreamcast.h"
/*
typedef	int	u_quad_t;	// quads
typedef	int	quad_t;

void PNGAPI 	png_set_write_fn (png_structp png_ptr, png_voidp io_ptr, png_rw_ptr write_data_fn, png_flush_ptr output_flush_fn)
{}
void 	png_flush (png_structp png_ptr){}
int png_write_data(const char *filename, uint8 *data, uint32 width, uint32 height){}
quad_t  __divdi3(quad_t a,quad_t b){}
long long __fixsfdi(float x){}

*/




int main( int argc, char *argv[] )
{

    char * filename = NULL, dcbname[ __MAX_PATH ], *ptr ;
    int i, j, ret = -1;
    file * fp = NULL;
    INSTANCE * mainproc_running;
    dcb_signature dcb_signature;

 	//DREAMCAST specific code!!
#if 1
    filename = "/cd/main.dcb";
	if (!file_exists(filename)) {
	    printf("Colombian Developers. \n");
	    printf("%s doesn't exist, quitting. \n",filename);
	    return -1;
	}

    // Remember to compile DCB with debug (bgdc -g) info!
    //debug = 1;
#endif


	/* Init SDL */

    SDL_DC_ShowAskHz(SDL_FALSE);
    SDL_DC_Default60Hz(SDL_FALSE);
    SDL_DC_VerticalWait(SDL_FALSE);
    SDL_DC_SetVideoDriver(SDL_DC_DIRECT_VIDEO);

	if ( SDL_Init (SDL_INIT_VIDEO | SDL_INIT_JOYSTICK | SDL_INIT_CDROM) < 0 )
	{
		printf ("ColDev - SDL Init Error: %s\n", SDL_GetError()) ;
		return 1 ;
	}

	//net_init();

	SDL_JoystickOpen(0);
	SDL_JoystickOpen(1);
	
    SDL_JoystickEventState(SDL_ENABLE);
    SDL_ShowCursor(SDL_ENABLE);
    SDL_DC_EmulateMouse(SDL_TRUE);
    SDL_DC_EmulateKeyboard(SDL_TRUE);


	SDL_DC_MapKey(0, SDL_DC_START, SDLK_RETURN);
	SDL_DC_MapKey(0, SDL_DC_A, SDLK_s);
	SDL_DC_MapKey(0, SDL_DC_B, SDLK_d);
	SDL_DC_MapKey(0, SDL_DC_X, SDLK_a);
	SDL_DC_MapKey(0, SDL_DC_Y, SDLK_w);
	SDL_DC_MapKey(0, SDL_DC_LEFT, SDLK_LEFT);
	SDL_DC_MapKey(0, SDL_DC_RIGHT, SDLK_RIGHT);
	SDL_DC_MapKey(0, SDL_DC_UP, SDLK_UP);
	SDL_DC_MapKey(0, SDL_DC_DOWN, SDLK_DOWN);
	SDL_DC_MapKey(0, SDL_DC_L, SDLK_ESCAPE);
	SDL_DC_MapKey(0, SDL_DC_R, SDLK_SPACE);

	SDL_DC_MapKey(1, SDL_DC_A, SDLK_g);
	SDL_DC_MapKey(1, SDL_DC_B, SDLK_h);
	SDL_DC_MapKey(1, SDL_DC_X, SDLK_f);
	SDL_DC_MapKey(1, SDL_DC_Y, SDLK_t);
	SDL_DC_MapKey(1, SDL_DC_LEFT, SDLK_KP4);
	SDL_DC_MapKey(1, SDL_DC_RIGHT, SDLK_KP6);
	SDL_DC_MapKey(1, SDL_DC_UP, SDLK_KP8);
	SDL_DC_MapKey(1, SDL_DC_DOWN, SDLK_KP2);

    /* Initialization (modules needed before dcb_load) */

    string_init() ;
    init_c_type() ;

    /* Init application title for windowed modes */

    strcpy( dcbname, filename ) ;

    if ( appname && filename != appname )
    {
        free( appname );
        appname = strdup( filename ) ;
    }

    if ( !embedded )
    {
        /* First try to load directly (we expect myfile.dcb) */
        if ( !dcb_load( dcbname ) )
        {
            char ** dcbext = dcb_exts;
            int dcbloaded = 0;

            while ( dcbext && *dcbext )
            {
                strcpy( dcbname, filename ) ;
                strcat( dcbname, *dcbext ) ;
                if (( dcbloaded = dcb_load( dcbname ) ) ) break;
                dcbext++;
            }

            if ( !dcbloaded )
            {
                printf( "%s: doesn't exist or isn't version %d DCB compatible\n", filename, DCB_VERSION >> 8 ) ;
                return -1 ;
            }
        }
    }
    else
    {
        dcb_load_from( fp, dcbname, dcb_signature.dcb_offset );
    }

    /* If the dcb is not in debug mode */

    if ( dcb.data.NSourceFiles == 0 ) debug = 0;

    /* Initialization (modules needed after dcb_load) */

    sysproc_init() ;

#ifdef _WIN32
    HWND hWnd = /*GetForegroundWindow()*/ GetConsoleWindow();
    DWORD dwProcessId;
    GetWindowThreadProcessId( hWnd, &dwProcessId );
    if ( dwProcessId == GetCurrentProcessId() ) ShowWindow( hWnd, SW_HIDE );
#endif

    argv[0] = filename;
    bgdrtm_entry( argc, argv );

    if ( mainproc )
    {
        mainproc_running = instance_new( mainproc, NULL ) ;
        ret = instance_go_all() ;
    }

#ifdef TARGET_PSP
    sceKernelExitGame();
#endif

    bgdrtm_exit( ret );

    free( appexename        );
    free( appexepath        );
    free( appexefullpath    );
    free( appname           );

    return ret;
}

